#!/usr/bin/env python
#Copyright 2008 David D Lowe
#released under the GPLv3
"""A pack is a Pigment like file that contains multiple pigments
However, it is not strictly a pigment
It's main purpose is to distribute several pigments easily. For example, 
you could include a skin and all of it's pigment dependancies in a pack"""

from __future__ import division
import sys, os
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.insert(0,"..")
from pigment import PigmentType, Pigment
import pigment
import const
from const import PIGMENT_TYPES
import ep_exceptions
import managepigments
from shell import SubprocessShell

class PackType(PigmentType):
    
    humanName = {"en":"Pack"}
    humanPluralName = {"en":"Packs"}
    codeName = "pack"
    abbreviation = "pck"
    directoryName = "packs"
    canGeneratePreview = False
    
    def get_Pigment(self):
        return Pack

class Pack():
    
    def __init__(self):
        self.type = PackType()
        self.codeName = ""
        self.humanName = {"en":""}
        self.ownPigments = []
    
    def __str__(self):
        return "Pack object named " + self.get_human_name() + " codename " + self.codeName
    
    def get_human_name(self, language="en"):
        return self.humanName[language]
    
    def set_human_name(self, humanName, language="en"):
        self.humanName[language] = humanName
    
    def get_state(self):
        """Pack's are either STATE_EMPTY, STATE_INFO or STATE_INCOMPLETE"""
        # TODO: to be redone, add states?
        print "get_state; codeName: %s humanName %s ownPigments %s" % (self.codeName, self.humanName, self.ownPigments)
        if self.codeName == "" and self.humanName == {"en":""} and len(self.ownPigments) == 0:
            return pigment.STATE_EMPTY
        elif self.codeName != "" and self.humanName != {"en":""} and len(self.ownPigments) != 0:
            return pigment.STATE_INFO
        else:
            return pigment.STATE_INCOMPLETE
    
    def debug_get_state_str(self):
        state = self.get_state()
        if state == pigment.STATE_EMPTY: return "STATE_EMPTY"
        elif state == pigment.STATE_INFO: return "STATE_INFO"
        elif state == pigment.STATE_INCOMPLETE: return "STATE_INCOMPLETE"
        else:
            return "Unknown state for pigment: %d" % state
    
    @staticmethod
    def __extract_in_temp(filePath, shell=None):
        """Extracts the downloaded pigment to /tmp/epidermis/installer
        shell is the Shell object that should be used
        Not the same as the Pigment object's private method"""
        if not os.path.exists(filePath):
            raise(ep_exceptions.FileNotFoundException(filePath))
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        fileName = filePath.split("/")[-1]
        shell.do(["rm", "-r","-f", "/tmp/epidermis/pack"])
        shell.do(["mkdir", "-p", "-m", "777", "/tmp/epidermis/pack"])
        shell.do(["cp", "-f", filePath, "/tmp/epidermis/pack/"])
        shell.do(["mv", "/tmp/epidermis/pack/%s" % fileName, "/tmp/epidermis/pack/%s.tar.gz" % fileName[:-8]])
        shell.do(["gunzip", "%s.tar.gz" % fileName[:-8]], cwd="/tmp/epidermis/pack")
        shell.do(["tar", "-xf", "%s.tar" % fileName[:-8]], cwd="/tmp/epidermis/pack")
    
    def install(self, installationDirectory, progressCallback=None, shell=None):
        """Installs all of the pack's owned pigments, leaving the skins
        until last"""
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        if progressCallback is None: progressCallback = lambda progress, query: False
        
        if progressCallback(0.0): return
        
        
        progress = 0
        # install all non-skin pigments first
        for pt in PIGMENT_TYPES[1:]:
            for pigm in self.ownPigments:
                if pigm.type.codeName == pt:
                    pigm.install(installationDirectory, shell=shell)
                    progress += 1
                    if progressCallback(progress / len(self.ownPigments)): return
        # now skins
        for pigm in self.ownPigments:
            if pigm.type.codeName == "skin":
                pigm.install(installationDirectory, shell=shell)
                progress += 1
                if progressCallback(progress / len(self.ownPigments)): return
        
        if len(self.ownPigments) == 0:
            progressCallback(1.0)
    
    def load(self, filePath):
        """Loads the pack and its owned Pigment objects"""
        self.__extract_in_temp(filePath)
        self.read("/tmp/epidermis/pack/information/pigment.xml")
        # non skins first
        for pt in PIGMENT_TYPES[1:]:
            for ff in self.pigmentsData[pt]:
                if os.path.exists(os.path.join("/tmp/epidermis/pack/data", ff)):
                    self.ownPigments.append(managepigments.create_pigment(pt))
                    self.ownPigments[-1].load(os.path.join("/tmp/epidermis/pack/data/", ff))
                else:
                    raise(ep_exceptions.MissingFileInPigment(self, ff))
        # now skins
        for ff in self.pigmentsData["skin"]:
            if os.path.exists(os.path.join("/tmp/epidermis/pack/data", ff)):
                self.ownPigments.append(managepigments.create_pigment("skin"))
                self.ownPigments[-1].load_with_pigments(os.path.join("/tmp/epidermis/pack/data/", ff), self.ownPigments)
            else:
                raise(ep_exceptions.MissingFileInPigment(self, ff))
    
    def read(self, filePath):
        """Reads the XML file passed
        Note: doesn't load pigment objects"""
        if not os.path.exists(filePath):
            raise(ep_exceptions.FileNotFoundException(file))
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        type = pigment.get_first_text(root.getElementsByTagName("type")[0])
        if type.lower().strip() != "pack":
            print >> sys.stderr, "Pack object loading non pack pigment, " + filePath
        self.type = PackType()
        
        for nn in root.getElementsByTagName("name"):
            if nn.hasAttribute("lang"):
                lang = nn.getAttribute("lang")
            else:
                lang = ""
            self.humanName[lang] = pigment.get_first_text(nn)
        
        self.codeName = pigment.get_first_text(root.getElementsByTagName("codename")[0])
        
        data = root.getElementsByTagName("data")[0]
        
        self.pigmentsData = {}
        for pt in PIGMENT_TYPES:
            self.pigmentsData[pt] = []
        for pigm in data.getElementsByTagName("pigment"):
            type = pigment.get_first_text(pigm.getElementsByTagName("type")[0])
            if type not in PIGMENT_TYPES:
                print >> sys.stderr, "unrecongised type: " + type
                break
            filename = pigment.get_first_text(pigm.getElementsByTagName("filename")[0])
            self.pigmentsData[type].append(filename)
            
